from os import path


def version_add(version1, step=10):
    version2 = version1[0] * step * step + version1[1] * step + version1[2] + 1

    version1[2] = version2 % step
    version1[1] = int(version2 / step) % step
    version1[0] = int(version2 / step / step)

    return version1


def get_version(argv, version_path, step=10):
    if path.exists(version_path):
        with open(version_path, 'r') as f:
            version1 = [int(i) for i in f.read().split('.')]
    else:
        version1 = [0, 0, 1]

    if len(argv) >= 2 and argv[1] == 'build':
        version1 = version_add(version1, step)

    version3 = '{}.{}.{}'.format(*version1)
    with open(version_path, 'w') as f:
        f.write(version3)
    return version3



import subprocess
import sys
from typing import List


def run_shell(command: str, printf=True) -> str:
    """
    执行shell命令
    """
    if printf:
        cmd = subprocess.Popen(command, stdin=subprocess.PIPE, stderr=sys.stderr, close_fds=True,
                               stdout=sys.stdout, universal_newlines=True, shell=True, bufsize=1)

        cmd.communicate()
        return cmd.returncode
    else:
        outputs = subprocess.Popen(
            command, stdout=subprocess.PIPE, shell=True).communicate()
        outputs = [i.decode('utf-8')[:-1] for i in outputs if i is not None]
        outputs = [i for i in outputs if len(i.strip()) > 0]
        return ''.join(outputs)


def run_shell_list(command_list: List[str], printf=True) -> str:
    """
    批量执行shell命令
    """
    # for shell in shell_list:
    #    run_shell(shell, printf=printf)
    command = ' && '.join(command_list)
    return run_shell(command, printf=printf)



"""
打包的工具类
"""
import argparse
import logging

from notebuild.shell import run_shell, run_shell_list

logger = logging.getLogger("notebuild")


class PackageBuild:
    """
    打包的工具类
    """

    def __init__(self, name=None):
        res = run_shell("git rev-parse --show-toplevel", printf=False)
        self.name = name or res.split('/')[-1]

    def git_pull(self):
        """
        git pull
        """
        logger.info("{} pull".format(self.name))
        run_shell("git pull")

    def git_push(self):
        """
        git push
        """
        logger.info("{} push".format(self.name))
        run_shell_list([
            'git add -A',
            'git commit -a -m "add"',
            'git push'
        ])

    def git_install(self):
        """
        git install
        """
        logger.info('{} install'.format(self.name))
        run_shell_list([
            'pip uninstall {} -y'.format(self.name),
            'python3 setup.py install',
            'rm -rf {}.egg-info'.format(self.name),
            'rm -rf dist',
            'rm -rf build'
        ])
        self.git_clear_build()

    def pip_install(self):
        """
        pip install
        """

        a = run_shell("git remote -v", printf=False)
        git_url = a.split('\n')[0].split('\t')[1].split(' ')[0]
        run_shell_list(['pip install -U -q git+{}'.format(git_url), ])
        logger.info('pip install {} success'.format(self.name))

    def git_clear_build(self):
        logger.info('{} build clear'.format(self.name))
        run_shell_list([
            'rm -rf {}.egg-info'.format(self.name),
            'rm -rf dist',
            'rm -rf build',
        ])

    def git_build(self):
        """
        git build
        """
        logger.info('{} build'.format(self.name))
        self.git_pull()
        self.git_clear_build()
        run_shell_list([
            'python3 setup.py build',  # 编译
            'python3 setup.py sdist',  # 生成 tar.gz
            'python3 setup.py bdist_egg',  # 生成 egg 包
            'python3 setup.py bdist_wheel',  # 生成 wheel 包

            # twine register dist/*
            'twine upload dist/*'  # 发布包
        ])
        self.git_clear_build()
        self.git_push()
        self.pip_install()

    def git_clean_history(self):
        """
        git build
        """
        logger.info('{} clean history'.format(self.name))
        run_shell_list([
            'git checkout --orphan latest_branch',  # 1.Checkout
            'git add -A',  # 2. Add all the files
            'git commit -am "clear history"',  # 3. Commit the changes
            'git branch -D master',  # 4. Delete the branch
            'git branch -m master',  # 5.Rename the current branch to master
            'git push -f origin master',  # 6.Finally, force update your repository
        ])

    def git_clean(self):
        """
        git clean
        """
        logger.info('{} clean'.format(self.name))
        run_shell_list([
            'git rm -r --cached .',
            'git add .',
            "git commit -m 'update .gitignore'",
            'git gc --aggressive'
        ])


def command_line_parser():
    parser = argparse.ArgumentParser(description="Test")
    parser.add_argument('command')
    args = parser.parse_args()
    return args


def notebuild():
    """
    build tool
    """
    args = command_line_parser()
    package = PackageBuild()
    if args.command == 'pull':
        package.git_pull()
    elif args.command == 'push':
        package.git_push()
    elif args.command == 'install':
        package.git_install()
    elif args.command == 'build':
        package.git_build()
    elif args.command == 'clean':
        package.git_clean()
    elif args.command == 'clean_history':
        package.git_clean_history()



import sys
from os import path

from setuptools import find_packages, setup


def version_add(version1, step=10):
    version2 = version1[0] * step * step + version1[1] * step + version1[2] + 1

    version1[2] = version2 % step
    version1[1] = int(version2 / step) % step
    version1[0] = int(version2 / step / step)

    return version1


def get_version(argv, version_path, step=10):
    if path.exists(version_path):
        with open(version_path, 'r') as f:
            version1 = [int(i) for i in f.read().split('.')]
    else:
        version1 = [0, 0, 1]

    if len(argv) >= 2 and argv[1] == 'build':
        version1 = version_add(version1, step)

    version3 = '{}.{}.{}'.format(*version1)
    with open(version_path, 'w') as f:
        f.write(version3)
    return version3


version_path = path.join(path.abspath(
    path.dirname(__file__)), 'script/__version__.md')

version = get_version(sys.argv, version_path, step=32)

install_requires = []

setup(name='notebuild',
      version=version,
      description='notebuild',
      author='niuliangtao',
      author_email='1007530194@qq.com',
      url='https://github.com/1007530194',
      packages=find_packages(),
      entry_points={'console_scripts': [
          'notebuild = notebuild.core:notebuild', ]},
      install_requires=install_requires,
      )
